@@ -16,7 +16,6 @@ except ImportError:
 import hmac
 import json
 import os, requests
-import pwd
 import datetime
 import pkg_resources
 import sys, time
@@ -151,6 +150,7 @@ class BeaconAPI(object):
                     raise BeaconAPIException(500, "Invalid local API key")
 
                 # convert key to API key
+                import pwd
                 username = pwd.getpwuid(os.getuid()).pw_name
                 self.key = 'local %s %s' % (username, six.ensure_str(
                     hmac.new(six.ensure_binary(ekey), six.ensure_binary(username), digestmod=hashlib.md5).hexdigest()))
@@ -237,5 +237,44 @@ class BeaconAPI(object):
     def put(self, _name, **args):      return self.do('put', _name, args)
     def delete(self, _name, **args):   return self.do('delete', _name, args)
 
+    def do_async(self, method, _name, args, use_tqdm=False, update_delay=1):
+        """
+        execute API call which is expected to return a polling object for progress
+        Args:
+            method: get, post, delete, put
+            _name:  endpoint name relative to /api/v1/
+            args:   dictionary of arguments
+            use_tqdm: show progress bar using TQDM. Recommended value: os.isatty(sys.stdin.fileno())
+            update_delay: number of seconds to wait before asking for progress
+        """
+        status = self.do(method, _name, args)
+        ident = status['ident']
+        if use_tqdm:
+            import tqdm
+            progress_bar = tqdm.tqdm(leave=False)
+        else:
+            progress_bar = None
+
+        try:
+            while 1:
+                progress = self.get('status', id=ident)
+                if progress['status'] == 'finished':
+                    return progress['extra']
+                elif progress['status'] == 'cancelled':
+                    raise BeaconAPIException(code=400, message="Cancelled")
+                elif progress['status'] == 'error':
+                    raise BeaconAPIException(code=400, message=progress['result'])
+                else:
+                    if progress_bar is not None:
+                        progress_bar.total = progress['total']
+                        progress_bar.n = progress['progress']
+                        progress_bar.desc = progress['task']
+                        progress_bar.refresh()
+                time.sleep(update_delay)
+        except KeyboardInterrupt:
+            self.delete('status', id=ident)
+            raise
+        finally:
+            if progress_bar is not None: progress_bar.close()
 
 api = BeaconAPI()
